==> kxmleditor-0.7.2.tar.gz: kxmleditor-0.7.2/part/kxe_treeviewitem.cpp <==
/***************************************************************************
                          kxe_treeviewitem.cpp  -  description
                             -------------------
    begin                : Wed Nov 21 2001
    copyright            : (C) 2001 by The KXMLEditor Team
    email                : lvanek@eanet.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxe_treeviewitem.h"
#include "qdom_add.h"

#include <qregexp.h>

#include <klistview.h>
#include <kdebug.h>
#include <kiconloader.h>

const QPixmap KXE_TreeViewItem::s_pmBookmark = QPixmap(SmallIcon("bookmark"));
unsigned int KXE_TreeViewItem::s_iDefaultExpandLevel = 0;
unsigned int KXE_TreeViewItem::s_iElemDisplayMode = 0;

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pParent )
	: QListViewItem(pParent),
	  m_xmlNode(xmlNode),
	  m_pPrevSibling(0),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, KListView * pParent )
	: QListViewItem(pParent),
	  m_xmlNode(xmlNode),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pParent, QListViewItem * pAfter )
	: QListViewItem( pParent, pAfter ),
	  m_xmlNode(xmlNode),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

void KXE_TreeViewItem::init()
{
	// inform the next sibling (if there's any) about this item (as its previous sibling)
	KXE_TreeViewItem * pNextItem = static_cast <KXE_TreeViewItem*> (nextSibling());
	if ( pNextItem )
		pNextItem->setPrevSibling(this);

	setPixmap( 0, domTool_getIconForNodeType(m_xmlNode.nodeType()) );

	if ( domTool_getLevel(m_xmlNode) < s_iDefaultExpandLevel )
		setOpen(true);

	setTexts();
}

void KXE_TreeViewItem::setTexts()
{
	switch ( m_xmlNode.nodeType() )
	{
		case QDomNode::ElementNode:

			setText( 0, m_xmlNode.toElement().tagName() );
			if ( s_iElemDisplayMode == 0 )
				setText( 1, QString() );
			else
			{
				// parse all attributes to fill the second column
				QString str2ndCol;
				for ( uint i=0; i < m_xmlNode.toElement().attributes().length(); i++ )
				{
					if ( i > 0 )
						str2ndCol += ", ";
					str2ndCol += m_xmlNode.toElement().attributes().item(i).toAttr().name();
					if ( s_iElemDisplayMode == 2 )
						str2ndCol += '=' + m_xmlNode.toElement().attributes().item(i).toAttr().value();
				}
				setText( 1, str2ndCol );
			}
			break;

		case QDomNode::TextNode:
		case QDomNode::CDATASectionNode:
		case QDomNode::CommentNode:
		{
			// set name
			QString strText = m_xmlNode.toCharacterData().data();
			strText = strText.replace( QRegExp("\n"), " " ); // replace every newline by a space
			strText = strText.replace( QRegExp("\t"), "" ); // removes every tab
			strText = strText.replace( QRegExp("\r"), "" ); // removes every return
			strText = strText.simplifyWhiteSpace();
			if( strText.length() > 30 ) // reduce name length, if necessary
				strText = strText.left(30) + "...";
			setText( 0, strText );

			break;
		}

		case QDomNode::ProcessingInstructionNode:

			setText( 0, m_xmlNode.toProcessingInstruction().target() + " " + m_xmlNode.toProcessingInstruction().data() );
			break;

		default:
			kdDebug() << "KXE_TreeViewItem::init: unknown node type (" << m_xmlNode.nodeType() << ")" << endl;
	}
}

void KXE_TreeViewItem::initChilds()
{
	QDomNode tmpNode = m_xmlNode.lastChild();

	while ( ! tmpNode.isNull() )
	{
		new KXE_TreeViewItem( tmpNode, this );
		tmpNode = tmpNode.previousSibling();		
	}
}

bool KXE_TreeViewItem::toggleBookmark()
{
	m_bBookmarked = ! m_bBookmarked;

	if (m_bBookmarked)
		setPixmap( 1, s_pmBookmark );
	else
		setPixmap( 1, QPixmap() );

	return m_bBookmarked;
}

KXE_TreeViewItem * KXE_TreeViewItem::lastChild() const
{
	// take the first child
	QListViewItem * pTmpItem = firstChild();
	// if there are no childs return 0
	if ( ! pTmpItem )
		return 0;

	while ( pTmpItem->nextSibling() )     // find last child by
		pTmpItem = pTmpItem->nextSibling(); // traversing all childs

	return static_cast <KXE_TreeViewItem*> (pTmpItem);
}

KXE_TreeViewItem * KXE_TreeViewItem::prevItem() const
{
	if ( m_pPrevSibling ) // if there is a prev. sibling
	{                     // return its last grand child (if there is any)
		KXE_TreeViewItem * pPrevItem = m_pPrevSibling;
		KXE_TreeViewItem * pTmpItem;
		while ( pTmpItem=pPrevItem->lastChild() )
			pPrevItem = pTmpItem;
		return pPrevItem;
	}
	else                                                 // if there is no prev. sibling,
		return static_cast <KXE_TreeViewItem*> (parent()); // return this' parent (if there is any)
}

KXE_TreeViewItem * KXE_TreeViewItem::nextItem()
{
	// checking for a child
	QListViewItem * pTmp = firstChild();
	if (pTmp)
		return static_cast <KXE_TreeViewItem*> (pTmp);

	// there is no child -> checking for the next sibling
	pTmp = nextSibling();
	if (pTmp)
		return static_cast <KXE_TreeViewItem*> (pTmp);

	// there is no next sibling -> checking for parents' next sibling(s)
	QListViewItem * pParent = parent();
	while (pParent)
	{
		pTmp = pParent->nextSibling();
		if (pTmp)
			return static_cast <KXE_TreeViewItem*> (pTmp);
		pParent = pParent->parent();
	}
	return 0;
}

void KXE_TreeViewItem::expandSubTree( int iLevel )
{
	setOpen(true); // expand this item

	if ( iLevel == 0 ) // return, if we are deep enough
		return;
	
	// expand childs (recursive)
	int iNewLevel = ( iLevel == -1 ) ? -1 : iLevel-1;
	KXE_TreeViewItem * pChild = static_cast <KXE_TreeViewItem*> ( firstChild() );
	while ( pChild )
	{
		pChild->expandSubTree(iNewLevel);
		pChild = static_cast <KXE_TreeViewItem*> ( pChild->nextSibling() );
	}
}

void KXE_TreeViewItem::collapseSubTree( int iLevel )
{
	if ( iLevel < 0 )
	{
		kdDebug() << "KXE_TreeViewItem::collapseSubTree: wrong level given (iLevel=" << iLevel << ")" << endl;
		return;
	}

	int iNewLevel;
	if (iLevel==0)     // collapse this item,
	{                  // because we are deep enough
		setOpen(false);
		iNewLevel = 0;
	}
	else
		iNewLevel = iLevel - 1;

	// collapsing in childs (recursive)
	KXE_TreeViewItem * pChild = static_cast <KXE_TreeViewItem*> ( firstChild() );
	while ( pChild )
	{
		pChild->collapseSubTree( iNewLevel );
		pChild = static_cast <KXE_TreeViewItem*> ( pChild->nextSibling() );
	}
}

==> kxmleditor_1.0.0.orig.tar.gz: kxmleditor-1.0.0/part/kxe_treeviewitem.cpp <==
/***************************************************************************
                          kxe_treeviewitem.cpp  -  description
                             -------------------
    begin                : Wed Nov 21 2001
    copyright            : (C) 2001, 2002, 2003 by The KXMLEditor Team
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxe_treeviewitem.h"

#include "kxmleditorfactory.h"
#include "kxeconfiguration.h"
#include "kxetreeviewsettings.h"

#include "qdom_add.h"

#include <qregexp.h>

#include <klistview.h>
#include <kdebug.h>
#include <kiconloader.h>

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pParent )
	: QListViewItem(pParent),
	  m_xmlNode(xmlNode),
	  m_pPrevSibling(0),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, KListView * pParent,  QListViewItem* pAfter)
	: QListViewItem(pParent,pAfter),
	  m_xmlNode(xmlNode),
    m_pPrevSibling(0),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

KXE_TreeViewItem::KXE_TreeViewItem( const QDomNode & xmlNode, QListViewItem * pParent, QListViewItem * pAfter )
	: QListViewItem( pParent, pAfter ),
	  m_xmlNode(xmlNode),
    m_pPrevSibling(0),
	  m_bBookmarked(false)
{
	init();
	initChilds();
}

KXE_TreeViewItem::~KXE_TreeViewItem()
{
  // inform the next sibling (if there's any) about destroying this item (as its previous sibling)
	KXE_TreeViewItem * pNextItem = static_cast <KXE_TreeViewItem*> (nextSibling());
	if ( pNextItem )
		{ pNextItem->setPrevSibling(m_pPrevSibling);
    }
}

void KXE_TreeViewItem::init()
{
	// inform the next sibling (if there's any) about this item (as its previous sibling)
	KXE_TreeViewItem * pNextItem = static_cast <KXE_TreeViewItem*> (nextSibling());
	if ( pNextItem )
		pNextItem->setPrevSibling(this);

	setPixmap(0, domTool_getIconForNodeType(m_xmlNode.nodeType(), false));

	if ( domTool_getLevel(m_xmlNode) < (unsigned int)KXMLEditorFactory::configuration()->treeview()->dfltExpLevel() )
		setOpen(true);

	setTexts();
}

void KXE_TreeViewItem::setTexts()
{
	switch ( m_xmlNode.nodeType() )
	{
		case QDomNode::ElementNode:

			setText( 0, m_xmlNode.toElement().nodeName() );

			if ( KXMLEditorFactory::configuration()->treeview()->elemDisplMode() == KXETreeViewSettings::NoAttributes )
				setText( 1, QString() );
			else
			{
				// parse all attributes to fill the second column
				QString str2ndCol;
				for ( uint i=0; i < m_xmlNode.toElement().attributes().length(); i++ )
				{
					if ( i > 0 )
						str2ndCol += ", ";
					str2ndCol += m_xmlNode.toElement().attributes().item(i).toAttr().name();
					if ( KXMLEditorFactory::configuration()->treeview()->elemDisplMode() == KXETreeViewSettings::NamesAndValues )
						str2ndCol += '=' + m_xmlNode.toElement().attributes().item(i).toAttr().value();
				}
				setText( 1, str2ndCol );
			}
			break;

		case QDomNode::TextNode:
		case QDomNode::CDATASectionNode:
		case QDomNode::CommentNode:
		{
			// set name
			QString strText = m_xmlNode.toCharacterData().data();
			strText = strText.replace( QRegExp("\n"), " " ); // replace every newline by a space
			strText = strText.replace( QRegExp("\t"), "" ); // removes every tab
			strText = strText.replace( QRegExp("\r"), "" ); // removes every return
			strText = strText.simplifyWhiteSpace();
			if( strText.length() > 30 ) // reduce name length, if necessary
				strText = strText.left(30) + "...";
			setText( 0, strText );

			break;
		}

		case QDomNode::ProcessingInstructionNode:

			setText( 0, m_xmlNode.toProcessingInstruction().target() );
			break;

		default:
			kdDebug() << "KXE_TreeViewItem::init: unknown node type (" << m_xmlNode.nodeType() << ")" << endl;
	}
}

void KXE_TreeViewItem::initChilds()
{
	QDomNode tmpNode = m_xmlNode.lastChild();

	while ( ! tmpNode.isNull() )
	{
		new KXE_TreeViewItem( tmpNode, this );
		tmpNode = tmpNode.previousSibling();		
	}
}

bool KXE_TreeViewItem::toggleBookmark()
{
	m_bBookmarked = ! m_bBookmarked;

	setPixmap(0, domTool_getIconForNodeType(m_xmlNode.nodeType(), m_bBookmarked));
	
	return m_bBookmarked;
}

KXE_TreeViewItem * KXE_TreeViewItem::lastChild() const
{
	// take the first child
	QListViewItem * pTmpItem = firstChild();
	// if there are no childs return 0
	if ( ! pTmpItem )
		return 0;

  QListViewItem * pTmpItem2;  
	while ( (pTmpItem2 = pTmpItem->nextSibling()) != 0 )     // traversing all childs
		{
      pTmpItem = pTmpItem2;
    }
    
	return static_cast <KXE_TreeViewItem*> (pTmpItem);
}

KXE_TreeViewItem * KXE_TreeViewItem::prevItem()
{
  if ( m_pPrevSibling ) // if there is a prev. sibling
	{                     // return its last grand child (if there is any)
		KXE_TreeViewItem * pPrevItem = m_pPrevSibling;
		KXE_TreeViewItem * pTmpItem;
		while ( (pTmpItem=pPrevItem->lastChild()) )
			pPrevItem = pTmpItem;
		return pPrevItem;
	}
	else                                                 // if there is no prev. sibling,
		return static_cast <KXE_TreeViewItem*> (parent()); // return this' parent (if there is any)
}

KXE_TreeViewItem * KXE_TreeViewItem::nextItem()
{
  // checking for a child
	QListViewItem * pTmp = firstChild();
	if (pTmp)
		return static_cast <KXE_TreeViewItem*> (pTmp);

	// there is no child -> checking for the next sibling
	pTmp = nextSibling();
	if (pTmp)
		return static_cast <KXE_TreeViewItem*> (pTmp);

	// there is no next sibling -> checking for parents' next sibling(s)
	QListViewItem * pParent = parent();
	while (pParent)
	{
		pTmp = pParent->nextSibling();
		if (pTmp)
			return static_cast <KXE_TreeViewItem*> (pTmp);
		pParent = pParent->parent();
	}
	return 0;
}

void KXE_TreeViewItem::expandSubTree( int iLevel )
{
	setOpen(true); // expand this item

	if ( iLevel == 0 ) // return, if we are deep enough
		return;
	
	// expand childs (recursive)
	int iNewLevel = ( iLevel == -1 ) ? -1 : iLevel-1;
	KXE_TreeViewItem * pChild = static_cast <KXE_TreeViewItem*> ( firstChild() );
	while ( pChild )
	{
		pChild->expandSubTree(iNewLevel);
		pChild = static_cast <KXE_TreeViewItem*> ( pChild->nextSibling() );
	}
}

void KXE_TreeViewItem::collapseSubTree( int iLevel )
{
	if ( iLevel < 0 )
	{
		kdDebug() << "KXE_TreeViewItem::collapseSubTree: wrong level given (iLevel=" << iLevel << ")" << endl;
		return;
	}

	int iNewLevel;
	if (iLevel==0)     // collapse this item,
	{                  // because we are deep enough
		setOpen(false);
		iNewLevel = 0;
	}
	else
		iNewLevel = iLevel - 1;

	// collapsing in childs (recursive)
	KXE_TreeViewItem * pChild = static_cast <KXE_TreeViewItem*> ( firstChild() );
	while ( pChild )
	{
		pChild->collapseSubTree( iNewLevel );
		pChild = static_cast <KXE_TreeViewItem*> ( pChild->nextSibling() );
	}
}

/** Test, if item in parameter is my direct or indirect child item */
bool KXE_TreeViewItem::isMyChildren(const KXE_TreeViewItem *pTestItem)
{
	KXE_TreeViewItem* pChildXmlTreeItem = (KXE_TreeViewItem*) firstChild();
  while(pChildXmlTreeItem)
		{ if(pChildXmlTreeItem == pTestItem)
				return true;

			// test child item childrens	
			if(pChildXmlTreeItem->isMyChildren(pTestItem))	
				return true;
				
			pChildXmlTreeItem = (KXE_TreeViewItem*) pChildXmlTreeItem->nextSibling();
    }
	return false;
}
